import zipfile
import xml.etree.ElementTree as ETree
import logging
from app.elements import element_factory, log_entry
import os
import os.path
import datetime
from config import RunningConfig


class Checker:
    """Class for package structure checking"""
    def __init__(self, package, structure):
        self.package = package
        self.structure = structure
        self.logs = {
            'errors': [],
            'warnings': [],
        }
        self.metadata = {}

        # configure logging
        logging.basicConfig(format='%(levelname)s: %(message)s',
                            level=logging.INFO)

    def validate(self):
        """Validates the package according to its structure structure"""
        self.metadata['package'] = os.path.basename(self.package)
        self.metadata['structure'] = os.path.basename(self.structure)
        self.metadata['start'] = datetime.datetime.now()
        logging.info("processing package {} at {}".format(
                     os.path.basename(self.package),
                     self.metadata['start'].strftime(RunningConfig.TIME_FORMAT)))

        try:
            zf = zipfile.ZipFile(self.package)
            logging.debug(zf.namelist())

            tree = ETree.parse(self.structure)
            root = element_factory(tree.getroot())
            root.validate(self.logs, zf)
        except zipfile.BadZipfile:
            log_entry('error', "this is not zip package!", self.logs)

        self.metadata['end'] = datetime.datetime.now()
        self.metadata['duration'] = self.metadata['end']-self.metadata['start']

        # formatting times for metadata
        self.metadata['end_f'] = self.metadata['end'].strftime(RunningConfig.TIME_FORMAT)
        self.metadata['start_f'] = self.metadata['start'].strftime(RunningConfig.TIME_FORMAT)
        self.metadata['duration_ts'] = self.metadata['duration'].total_seconds()

        logging.info("processing of {} finished at {} (duration: {} s)".format(
                     os.path.basename(self.package),
                     self.metadata['end_f'],
                     self.metadata['duration_ts']
                     ))

    def get_result(self):
        """Returns the result as dictionary"""
        return self.logs

    def get_metadata(self):
        """returns the check metadata"""
        return self.metadata