import logging
import sys
import re

__author__ = 'mirek'


def log_entry(level, msg, logs):
    level = level.lower()
    if level == 'error':
        logging.error(msg)
        logs['errors'].append(msg)
    elif level == 'warning':
        logging.warning(msg)
        logs['warnings'].append(msg)


def to_bool(var):
    """Converts string variable to boolean type"""
    return var.lower() in 'true'

def element_factory(element):
    """Factory method"""
    if re.search("(?i)directory$", element.tag):
        return Directory(element)
    elif re.search("(?i)file$", element.tag):
        return File(element)
    elif re.search("(?i)package$", element.tag):
        return Package(element)

class Element:
    def __init__(self, element):
        self.element = element
        self.attributes = element.attrib

        # check case sensitivity
        self.__case_sensitive = True
        if 'casesensitive' in self.attributes:
            self.__case_sensitive = to_bool(self.attributes['casesensitive'])

        self.name = self.attributes['name']

        self.pattern = self.name
        if self.attributes.get('pattern') is not None:
            self.pattern = self.attributes['pattern']

        # check mandatory
        self.__mandatory = True
        if 'mandatory' in self.attributes:
            self.__mandatory = to_bool(self.attributes['mandatory'])

    def is_mandatory(self):
        """docstring for isMandatory"""
        return self.__mandatory

    def get_name(self):
        """docstring for get_name"""
        return self.name

    def get_pattern(self):
        """docstring for get_pattern"""
        return self.pattern

    def get_children(self):
        """docstring for get_children"""
        return self.element.getchildren()

    def is_case_sensitive(self):
        """returns true, if case sensitive"""
        return self.__case_sensitive

    def validate(self, logs, package, parent=None, human_parent=None):
        logging.error('Unknown element {}'.format(self.get_name()))
        sys.exit(1)  # TODO must be changed according to the service


class Package(Element):
    """Class defining PACKAGE element"""

    def validate(self, logs, package):
        # check the package name
        if not re.search("(?i)" + self.get_name(), package.filename):
            log_entry('error',
                      "the name of the package is not in the correct format",
                      logs)

        # process the root directory
        for element in self.element:
            e = element_factory(element)
            e.validate(logs, package)


class Directory(Element):
    """Class defining DIRECTORY element"""
    def __init__(self, element):
        """docstring for __init__"""
        Element.__init__(self, element)
        self.name += '/'

        self.pattern = self.name
        if self.attributes.get('pattern') is not None:
            self.pattern = self.attributes['pattern'] + '/'

    def validate(self, logs, package, parent=None, human_parent=None):
        """Validates directory element"""
        logging.info("checking directory " + self.get_name())

        if parent is None:
            path = self.get_pattern()
            human_path = self.get_name()
        else:
            path = parent + self.get_pattern()
            human_path = human_parent + self.get_name()

        pattern = (path if self.is_case_sensitive() else "(?i)" + path) + ".*"

        # check if directory is in package
        in_package = False
        for a_file in package.namelist():
            # print( "{} vs {}".format( pattern, file))
            if re.search(pattern, a_file):
                in_package = True
                break

        if not in_package:
            msg = "directory '{}' is missing or not found at correct location."
            log_entry('error' if self.is_mandatory() else 'warning',
                      msg.format(human_path), logs)

        # walk through the content of the package
        for children in self.get_children():
            element = element_factory(children)
            element.validate(logs, package, path, human_path)


class File(Element):
    """Class defining FILE element"""
    def is_in_package(self, package):
        """docstring"""
        pass

    def validate(self, logs, package, parent=None, human_parent=None):
        # def checkFile(element, zf, errorLog, parent = None, humanParent = None):
        # prepare the pattern (combine it with path)
        logging.info("checking file " + self.get_name())

        if parent is None:
            path = self.get_pattern()
            human_path = self.get_name()
        else:
            path = parent + self.get_pattern()
            human_path = human_parent + self.get_name()

        pattern = '^' + (path if self.is_case_sensitive() else "(?i)" + path)+'$'

        # check if file is in package
        is_in_package = False
        a_file = None
        for a_file in package.namelist():
            # print("{} vs {}".format(pattern, file))
            if re.search(pattern, a_file):  # and element.isMandatory():
                is_in_package = True
                break

        # if file si not in package, display error message
        if not is_in_package:
            msg = "file '{}' is missing, not properly formatted or not found at correct location."
            log_entry('error' if self.is_mandatory() else 'warning',
                      msg.format(human_path), logs)
        else:
            # otherwise check the content if there is need to
            if len(self.get_children()) > 0:

                # check all of the CONTAINS elements
                for entry in self.get_children():
                    if entry.tag.endswith('contains'):
                        # get the line number if available
                        line_no = None
                        if 'line' in entry.keys():
                            line_no = entry.attrib['line']

                        zip_file = package.open(a_file)

                        is_match = False
                        counter = 0
                        for line in zip_file.read().splitlines():
                            counter += 1
                            print(str(counter) + " " + entry.attrib['pattern'])

                            match = re.match(entry.attrib['pattern'], str(line))
                            if match is not None and line_no != counter:
                                is_match = True
                                break

                            # if line number reached, don't need to go further
                            if line_no == counter:
                                break
                        if not is_match:
                            log_entry('error', "{}: {}".format(human_path,
                                      entry.attrib['message']), logs)

                        zip_file.close()
